# cryptoprotect
#
# TeslaCrypt encrypts files as long as the user has the permissions to do so.
# This script protects files using a DENY-ACL and can run on a windows file server as scheduled Task
#
# (C) 2016 Frank.carius@netatwork.de
#
# Version 1.0
#
param (
	[string]$basepath = "C:\group",     # share or path
	[string]$denygroup = "msxfaq\sgg-cryptoprotect",   # name of the group 
	[int]$idledays = 30,                            # days to wait on modified files
	[string]$reportcsvfile = (".\cryptprotect-" + (Get-Date -Format yyyyMMddhhmmss) + ".csv"),  # logfile for changes
	[string]$reportcsvfiledetail = "ProtectNew", #ProtectExist SkipDir Skip-LastWriteTimeUTC Skip-CreationTimeUTC", #detail level
	[switch]$whatif = $false                        # set switch to simulate only
)

set-psdebug -strict

write-host "cryptoprotect: Start"
write-host "cryptoprotect: Parameter basepath      :" $basepath
write-host "cryptoprotect: Parameter denygroup     :" $denygroup
write-host "cryptoprotect: Parameter idledays      :" $idledays
write-host "cryptoprotect: Parameter reportcsvfile :" $reportcsvfile
write-host "cryptoprotect: Parameter whatif        :" $whatif

#write-host "cryptoprotect: Parameter reportcsv:" $reportcsv
[datetime]$now = (get-date)

write-host " Verify group"
$denygroupPrincipal = New-Object System.Security.Principal.NTAccount($denygroup)
try {
	$denygroupPrincipal.Translate([System.Security.Principal.SecurityIdentifier]) | out-null
}
catch {
	write-error "  Unable to resolve group $denygroup"
	exit 1
}

Write-Host " Prepare Deny Entry"
$InheritanceFlag = [System.Security.AccessControl.InheritanceFlags]::None 
$PropagationFlag = [System.Security.AccessControl.PropagationFlags]::None 
$objType =[System.Security.AccessControl.AccessControlType]::Deny
$colRights = [System.Security.AccessControl.FileSystemRights]"Write, Delete" 
$objACE = New-Object System.Security.AccessControl.FileSystemAccessRule `
   ($denygroupPrincipal, $colRights, $InheritanceFlag, $PropagationFlag, $objType) 

write-host " Start processing files"
[uint32]$totalfiles=0
[uint32]$totalprotectedfiles=0
[uint32]$newprotectedfiles=0
get-childitem -path $basepath -recurse | %{
	$totalfiles++
	$result = "" | select timestamp,fullname,status
	$result.timestamp =  get-date -Format yyyyMMddHHmmss
	if ($totalfiles%1000 -eq 0) {
		write-host "Files:" $totalfiles
	}
    $fullname = [string]$_.Fullname
	write-host $fullname -nonewline
	$result.fullname = $fullname
	if ($_.PSIsContainer) {
		write-host "  SkipDir"
		$result.status="SkipDir"
	}
	else {
		if ((($now - $_.CreationTimeUtc).days) -lt $idledays) {
			Write-host " Skip-CreationTimeUTC"
			$result.status="SkipCreateTimeUTC"
		}
		elseif ((($now - $_.LastWriteTimeUtc).days) -lt $idledays) {
			Write-host " Skip-LastWriteTimeUTC"
			$result.status="SkipLastWriteTimeUTC"
		}
		else {
			$totalprotectedfiles++
			write-host " Protect-" -nonewline
			#$acl=get-acl $FullName
			#$acl = (Get-Item $path).GetAccessControl("Access")
            #$acl= $_.GetAccessControl("Access")   # hole nur die Access-Infos aus der ACL
			$acl= $_.GetAccessControl()   # hole die komplette ACL samt Owner etc
			
			if ($acl.access | where {$_.IdentityReference.Value -eq $denygroup}) {
				write-host "Exist"
				$result.status="ProtectExist"
			}
			else {
				write-host "Set-ACL"
				$result.status="ProtectNew"
				$acl.AddAccessRule($objACE)  # add ACL
				if (!$whatif) {
                    $_.SetAccessControl($acl)
					#set-acl -path $Fullname -aclobject $acl  # update File
				}
				$newprotectedfiles++
			}
		}
	}
	if ($reportcsvfiledetail.contains($result.status)) {
		if ($whatif) {
			$result.status = $result.status+"-whatif"
		}
		$result	
	}
} | export-csv $reportcsvfile -encoding unicode -notypeinformation

write-host " Total Files            : $totalfiles"
write-host " Total ProtectedFiles   : $totalprotectedfiles"
write-host " Total newprotectedfiles: $newprotectedfiles"
write-host "cryptoprotect: End"